import matplotlib.pyplot as plt
import logomaker
from ExpoSeq.tidy_data.tidy_seqlogoPlot import cleaning
import numpy as np

def plot_logo_single(ax, sequencing_report, sample, font_settings, highlight_specific_pos, highlight_pos_range, chosen_seq_length = 16):
    aa_distribution, sequence_length, length_filtered_seqs = cleaning(sample,
                                                                      sequencing_report,
                                                                      chosen_seq_length)
    logo_plot = logomaker.Logo(aa_distribution,
                               shade_below=.5,
                               fade_below=.5,
                               font_name='Arial Rounded MT Bold',
                               ax=ax,
                               )
    logo_plot.style_xticks(anchor=0,
                           spacing=1,
                           rotation=0)
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    plt.title("Logo Plot of " + sample + " with sequence length " + str(chosen_seq_length), **font_settings)
    font_settings["fontsize"] = original_fontsize
    if highlight_specific_pos != False:
        logo_plot.highlight_position(p=5,
                                     color='gold',
                                     alpha=.5)
 #   if highlight_pos_range != False:
  #      logo_plot.ax.highlight_position_range(pmin=3,
   #                                           pmax=5,
    #                                          color="lightcyan")


def plot_logo_multi(fig, sequencing_report, samples,num_cols, font_settings, chosen_seq_length = 16, test_version = False):
    if samples == "all":
        unique_experiments = sequencing_report["Experiment"].unique()
        unique_experiments = np.sort(unique_experiments)
    else:
        unique_experiments = sequencing_report["Experiment"].unique()
        unique_experiments = np.array([i for i in unique_experiments if i in samples])
        unique_experiments = np.sort(unique_experiments)
    Tot = unique_experiments.shape[0]
    Cols = num_cols
    # Compute Rows required
    Rows = Tot // Cols
    if Tot % Cols != 0:
        Rows += 1
    # Create a Position index
    Position = range(1, Tot + 1)
    n = 0
    adapted_fontsize = 10 - int(Cols) + 2
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = adapted_fontsize
  #  fig = plt.figure(1, constrained_layout=True)
    for i in unique_experiments:
        aa_distribution, sequence_length, length_filtered_seqs = cleaning(i,
                                                                          sequencing_report,
                                                                          chosen_seq_length)
        if length_filtered_seqs != 0:
            if length_filtered_seqs < 100:
                print("only " + str(length_filtered_seqs) + " sequences with the given length were found. The results might be biased")
            ax = fig.add_subplot(Rows,
                                 Cols,
                                 Position[n],
                                 xticks = (np.arange(0, chosen_seq_length, step = 1)))
            logo_plot = logomaker.Logo(aa_distribution,
                                        shade_below=.5,
                                        fade_below=.5,
                                        font_name='Arial Rounded MT Bold',
                                        ax=ax,
                                        )
            #logo_plot.set_xticks(range(aa_distribution.shape[0]))
            logo_plot.style_xticks(anchor=0,
                                   spacing=1,
                                   rotation=0,)
            plt.title(i, **font_settings) # check out https://matplotlib.org/3.1.0/api/_as_gen/matplotlib.pyplot.title.html


            n = n + 1
        else:
            print("Sample " + i + "was skipped because no sequence was found")
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    fig.suptitle("Logo Plots for sequence Length " + str(chosen_seq_length), **font_settings)
    font_settings["fontsize"] = original_fontsize





import re
import numpy as np
import pandas as pd

# output == number of aligned reads
# input == number of total sequencing reads

# create class depending on which data should be analyzed

class read_intermediate_reports:
    def __init__(self, filename):
        self.sequencing_report = filename
        #questions have to be otuside of class, otherwise they'll be asked each time a file is read

    def filter_rows_on_min(self, column, min_count):
        self.sequencing_report = self.sequencing_report[(self.sequencing_report[column] > min_count)]
        return self.sequencing_report

    def filter_rows_not_divisible(self, divisor, column):
        self.sequencing_report = self.sequencing_report[(self.sequencing_report[column] % divisor) == 0]
        return self.sequencing_report

    def get_file_length(self):
        length = self.sequencing_report.shape[0]
        return length

    def get_individuals(self, column):
        unique_values = self.sequencing_report[column].unique()
        return unique_values
    def sort_table(self, sort_params):
        self.sequencing_report = self.sequencing_report.sort_values(by = sort_params)
        return self.sequencing_report

    def nest_data(self, nest_by, column_to_list, unique):
        if unique == True:
            apply_fun = lambda x: pd.unique([z for z in x]).tolist()
        else:
            apply_fun = lambda x: list(x)
        self.sequencing_report = self.sequencing_report.groupby(nest_by)[column_to_list].agg(apply_fun).reset_index()
        return self.sequencing_report
    def sum_nest_data(self, nested_column_name, axis):
        column = self.sequencing_report[nested_column_name]
        clone_counts = np.array(column)
        summed_counts = []
        for nest in clone_counts:
            sum = np.sum(nest, axis = axis)
            summed_counts.append(sum)
        summed_counts = np.array(summed_counts)
        return summed_counts

    def max_row(self, column):
        length_nested_seq = []
        for nested in self.sequencing_report[column]:
            length_nested_seq.append(len(nested))
            length_nested_seq = np.array(length_nested_seq)
        max = max(length_nested_seq)
        return max

    def extract_substring_rows(self, lib_name, col_name = "Experiment"):
        sub_table = self.sequencing_report[self.sequencing_report[col_name].str.contains(lib_name)]
        return sub_table

    def summarize_duplicates(self, column_to_sum, duplicate_column, group):
        if group != '' or False:
            group.append(duplicate_column)
            group_columns = group
        else:
            group_columns = [duplicate_column]
        column = self.sequencing_report.groupby(group_columns)[column_to_sum].transform('sum')
        self.sequencing_report[column_to_sum] = column
        self.sequencing_report = self.sequencing_report.drop_duplicates(group_columns,
                                                                        keep = 'last')
        return self.sequencing_report

    def map_to_column(self, object_to_map, column_to_map, new_column_name):
        self.sequencing_report[new_column_name] = self.sequencing_report[column_to_map].map(object_to_map)
        return self.sequencing_report






class read_alignment_report:
    def __init__(self, alignment_report):
        self.report = alignment_report
        self.digit = "\d+"

    def find_word_align_rep(self, word, regex): #"Total sequencing reads"  # auomate this or ask user how it is called in his / her file
        for line in self.report:
            if line.find(word) != -1:
                preferred_line = line
                total_sequenced_reads = re.search(regex,
                                                preferred_line).group()
        return total_sequenced_reads






from ExpoSeq.tidy_data.tidy_stacked_aa_distr import cleaning
import matplotlib.pyplot as plt

def stacked_aa_distr(ax, sequencing_report, sample, region, protein, font_settings, legend_settings):
    aa_distribution = cleaning(sequencing_report, sample, region, protein)
    aa_distribution.plot(kind='bar', stacked=True, ax = ax)
    ax.set_xlabel('Position on amino acid sequence', **font_settings)
    if protein == True:
        ax.set_ylabel('Relatvie Proportion of Amino Acid', **font_settings)
    else:
        ax.set_ylabel('Relatvie Proportion of Nucleotide', **font_settings)

#    ax.set_xticks(rotation = 360)
    plt.legend(**legend_settings)
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    if protein == True:
        ax.set_title("Distribution of Aminoacids per Position for your sequences",pad = 12, **font_settings)
    else:
        ax.set_title("Distribution of Nucleotides per Position for your sequences",pad = 12, **font_settings)
    ax.set_xticklabels(ax.get_xticklabels(), rotation=90, ha='center')
    font_settings["fontsize"] = original_fontsize


from collections import Counter

import matplotlib.pyplot as plt
import numpy as np
from pandas import DataFrame, concat


sequencing_report = tidy_data.sequencing_report
third_round = ['EDE_K', 'ABA_K', 'ACA_K', 'FGF_L']
second_round = ['ED-_K', 'AB-_K', 'AC-_K', 'FG-_L']
first_round = ['E--_K', 'A--_K', 'F--_L']
rounds = [first_round, second_round, third_round]
width = 1/(len(rounds))
labels = ["First Round", "Second Round", "Third Round"]
n = 0
colors = ["orange", "royalblue", "limegreen"] # has to be changed
# works only for three panning rounds otherwise barplot will be confused
for round in rounds:
    sub_report = sequencing_report.loc[sequencing_report["Experiment"].isin(round)]
    sequences = list(sub_report["aaSeqCDR3"])
    counts = np.array(sub_report["cloneCount"])
    aminoacids = {"A": 0, "C": 0, "D": 0, "E": 0, "F": 0, "G": 0, "H": 0, "I": 0, "K": 0, "L": 0, "M": 0, "N": 0, "P": 0, "Q": 0, "R": 0, "S": 0, "T": 0,  "V": 0, "W": 0, "Y": 0}
    aa_count = DataFrame([aminoacids], columns = aminoacids.keys())

    for i in range(len(sequences)):
        aa = Counter(sequences[i])
        occurences = np.asarray(list(aa.values())) * counts[i]
        keys = aa.keys()
        local_dic = dict(zip(keys, occurences))
        local_dic = DataFrame([local_dic])
        aa_count = concat([aa_count, local_dic], axis =0).reset_index()
        aa_count.drop(columns = aa_count.columns[0], axis=1, inplace=True)
    summed_aa = aa_count.sum()
    summed_aa = np.array(summed_aa)
    normalized_aa = summed_aa/np.sum(summed_aa)
    aminoacids_keys = aa_count.columns
    X = np.arange(normalized_aa.shape[0])
    plt.bar(X + (0.3 - 0.3*n), normalized_aa, label = labels[n], width = 0.3, color = colors[n])
    n += 1
X = np.arange(normalized_aa.shape[0])
plt.xticks(X, aminoacids_keys)
plot_style = openParams('plot_style.txt')
plt.ylabel("relative Abundance", **plot_style)
plt.xlabel("Amino Acids", **plot_style)
params_legend = openParams("USQ_plot_legend_params.txt")
plt.legend(**params_legend)






sequences = sequencing_report_all['aaSeqCDR3'].apply(lambda x: list(x))
a = sequences.explode()


import pandas as pd

import os
try:
    import tkinter as tk
    from tkinter import filedialog
except:
    pass


def read_raw_data():
    try:
        filename = filedialog.askopenfilename()
    except:
        while True:
            filename = input("copy and paste the path to your file here")
            if os.path.isfile(filename) == True:
                break
            else:
                print("Please enter a valid filepath")
    filename = os.path.abspath(filename)
    pandas_raw_table = pd.read_table(filename)
    return pandas_raw_table


import pandas as pd
import csv

def load_mixed_files(filenames, testing, value = None):
    all_alignment_reports = pd.DataFrame([])
    sequencing_report = pd.DataFrame([])
    if not testing:
        column_choice = input("Please check if your report contains a column which contains the name of the samples. If the name of the column is Experiment you can skip this part by pressing 1. If it has another column name you can enter it here. If your files do not contain the column you have to enter the name of the experiment for each file individually. Therefore, press 3 to continue.")
        while True:
            if column_choice in ["1", '2', '3']:
                break
            else:
                print("Please enter a valid value")
                column_choice = input()
    else:
        column_choice = value
    for i in filenames:
        if "AlignmentReport" in i:
            report = pd.read_table(i)
            assert report.shape[1] == 1, "The alignment report should only one column. Please check your file."
            splitted_report = report.iloc[:, 0].str.split(":",
                                                          expand=True)
            splitted_report = splitted_report.iloc[:, 0:2].T
            transposed_report = splitted_report.rename(columns=splitted_report.iloc[0]).drop(splitted_report.index[0])
            all_alignment_reports = pd.concat([all_alignment_reports, transposed_report])
        else:
            try:
                with open(i, newline='') as f:
                    reader = csv.reader(f)
                    # The csv.reader object can read the file line by line
                    # If the file is not a CSV file, this will raise a csv.Error exception
                    # Otherwise, it will read the file successfully
                    for row in reader:
                        pass
                is_csv = True
            except:
                is_csv = False
            try:
                if is_csv == True:

                    local_intermediate_report = pd.read_csv(i)
                else:
                    local_intermediate_report = pd.read_table(i)

                while True:
                    if column_choice == "1":
                        break
                    elif column_choice == "3":
                        sample_name = input("Please enter the name of the sample for file " + i)
                        local_intermediate_report["Experiment"] = sample_name
                        break
                    else:
                        if column_choice in local_intermediate_report.columns:
                            local_intermediate_report = local_intermediate_report.rename(columns={column_choice: 'Experiment'})
                            break
                        else:
                            if not testing:
                                while True:
                                    column_choice = input("Apparently, you have entered the wrong column. Please enter the correct column name. If the name of the column is Experiment you can skip this part by pressing 1.")
                                    if column_choice == "1" or column_choice in local_intermediate_report.columns:
                                        break
                                    else:
                                        print("Please enter a valid value")
                            else: 
                                pass

                sequencing_report = pd.concat([sequencing_report, local_intermediate_report])
            except:
                print(i + " could not be read.")
    try:
        del all_alignment_reports["index"]
    except:
        pass
    return sequencing_report, all_alignment_reports





def load_alignment_reports(filenames):
    all_alignment_reports = pd.DataFrame()
    for i in filenames:
        if "AlignmentReport" in i:
            report = pd.read_table(i)
            splitted_report = report.iloc[:, 0].str.split(":",
                                                          expand = True)
            splitted_report = splitted_report.iloc[:, 0:2].T
            transposed_report = splitted_report.rename(columns=splitted_report.iloc[0]).drop(splitted_report.index[0])
            all_alignment_reports = pd.concat([all_alignment_reports, transposed_report])
    all_alignment_reports = all_alignment_reports.reset_index()
    try:
        del all_alignment_reports["index"]
    except:
        pass
    return all_alignment_reports


from .plots.usq_plot import plot_USQ
from .plots.plt_heatmap import plot_heatmap
from .plots.logo_plot import plot_logo_multi, plot_logo_single
from .plots.length_distribution import length_distribution_single, length_distribution_multi
from .plots.levenshtein_clustering import clusterSeq, cluster_single_AG
from .plots.embedding_with_binding import cluster_toxins_tsne
from .plots.relative_sequence_abundance import relative_sequence_abundance
from .plots.cluster_embedding import show_difference
from .plots.stacked_aa_distribution import stacked_aa_distr
from .plots.barplot import barplot
from .plots.saveFig import saveFig
import matplotlib.pyplot as plt
from .augment_data.binding_data import collect_binding_data
from .augment_data.uploader import upload
from ast import literal_eval
from .settings.plot_styler import PlotStyle
import pandas as pd
import pickle
import pkg_resources
import os
from ExpoSeq.settings.change_save_settings import Change_save_settings
from ExpoSeq.augment_data.randomizer import create_sequencing_report, create_binding_report
from ExpoSeq.reset import original_settings
import shutil

class PlotManager:
    def __init__(self, test_version = False, test_exp_num = 3, test_panrou_num = 1):
        self.is_test = test_version
        self.module_dir = os.path.abspath("")
        self.pkg_path = pkg_resources.resource_filename("ExpoSeq", "")
        common_vars_path = os.path.join(self.pkg_path,
                                        "settings",
                                        "global_vars.txt")
        if not os.path.isdir(os.path.join(self.module_dir, "my_experiments")):
            print("create my_experiments directory")
            os.mkdir(os.path.join(self.module_dir, "my_experiments"))
        if not os.path.isdir(os.path.join(self.module_dir, "temp")):
            print("create temp directory")
            os.mkdir(os.path.join(self.module_dir, "temp"))
            
        with open(common_vars_path, "r") as f:
            self.global_params = f.read()
        self.global_params = literal_eval(self.global_params)
        if test_version == True:
            if os.path.isdir(os.path.join(self.module_dir, "my_experiments")):
                pass
            else:
                os.mkdir(os.path.join(self.module_dir, "my_experiments"))
            if os.path.isdir(os.path.join(self.module_dir, "temp")):
                pass
            else:
                os.mkdir(os.path.join(self.module_dir, "temp"))
                
            self.experiment = "Test"
            self.sequencing_report = create_sequencing_report(num_experiments = test_exp_num,
                                                              panning_rounds = test_panrou_num,
                                                              )
            self.unique_experiments = self.sequencing_report["Experiment"].unique().tolist()
            self.unique_experiments = dict(zip(self.unique_experiments, self.unique_experiments))
            self.alignment_report = None
            self.sequencing_report["Experiment"] = self.sequencing_report["Experiment"].map(self.unique_experiments)
            self.binding_data = create_binding_report(self.sequencing_report,
                                                      num_antigen = test_panrou_num)
            
        else:
            self.sequencing_report, self.alignment_report, self.experiment = upload()
            experiment_path = os.path.join(self.module_dir,
                                            "my_experiments",
                                            self.experiment,
                                            "experiment_names.pickle")
            with open(experiment_path, "rb") as f:
                self.unique_experiments = pickle.load(f)
            self.sequencing_report["Experiment"] = self.sequencing_report["Experiment"].map(self.unique_experiments)
            self.add_binding = input("Do you have binding Data? Y/n")
            if self.add_binding.lower() in ["Y", "y"]:
                self.binding_data = collect_binding_data()
            else:
                self.binding_data = None
                
                
        font_settings_path = os.path.join(self.pkg_path,
                                          "settings",
                                          "font_settings.txt")
        assert os.path.isfile(font_settings_path), "The font settings file does not exist in the given filepath"
        with open(font_settings_path, "r") as f:
            font_settings = f.read()
        self.font_settings = literal_eval(font_settings)
        legend_settings_path = os.path.join(self.pkg_path,
                                            "settings",
                                            "legend_settings.txt")
        assert os.path.isfile(legend_settings_path), "The legend settings file does not exist in the given filepath"
        with open(legend_settings_path, "r") as f:
            legend_settings = f.read()
        colorbar_path = os.path.join(self.pkg_path,
                                     "settings",
                                     "colorbar.txt")
        assert os.path.isfile(colorbar_path), "The colorbar file does not exist in the given filepath"
        
        with open(colorbar_path, "r") as f:
            self.colorbar_settings = f.read()

        self.colorbar_settings = literal_eval(self.colorbar_settings)
        self.legend_settings = literal_eval(legend_settings)
        self.zero = 0
        self.batch_size = 300
        self.fig = plt.figure(1)
        self.ax = self.fig.gca()
        self.plot_type = "multi"
        self.style = PlotStyle(self.ax, self.plot_type)
        self.settings_saver = Change_save_settings()
        self.experiments_list = list(self.unique_experiments.values())
    def discard_samples(self, samples_to_discard):
        """

        :param samples_to_discard: the name of the samples in a list which you want to delete from your further analysis
        :return: updates the sequencing report where the values for the given samples do not exist anymore

        """
        assert type(samples_to_discard) == list, "You have to give a list with the samples you want to discard"
        self.sequencing_report = self.sequencing_report[~self.sequencing_report['Experiment'].isin(samples_to_discard)]

  #  def askMe(self):
   #     """
    #    :return: calls the chatbot which can help you to customize your plots or with other question in life and science.
     #   """
      #  askMe(self.global_params)

    def add_binding_data(self):
        self.binding_data = collect_binding_data(binding_data = self.binding_data)
    def print_antigens(self):
        """
        :return: prints the antigens (columns) of your binding data
        """
        try:
            print(self.binding_data.columns.to_list()[1:])
        except:
            print("You have not given binding data.")
    def print_samples(self):
        """
        :return: prints the names of your samples, so you can insert them in lists or similar for the analysis with some plots
        """
        print(list(self.unique_experiments.values()))
    def save(self, name = False):
        """
        :return: can be used to save your plots. It will ask you automatically for the directory where you want to save it
        """
        saveFig(name)
    def close(self):
        plt.close()

    def change_experiment_names(self, specific = None, change_whole_dic = False):
        """
        :param specific: optional parameter. You can use this function to change the names of a specific sample.
        :param change_whole_dic: optional Parameter. The renaming can be done by using a dictionary and map it to the labels. You can change multiple or all labels by adding the new dictionary for this parameter.
        :return:You can use this function to change the name of your samples. Thus, you can change the labels of your plots.
        """

        if specific != None:
            assert type(specific) == str, "You have to give a string (inside: "") as input for the specific sample you want to change"
        if change_whole_dic != False:
            assert type(change_whole_dic) == dict, "You have to give a dictionary as input for the specific sample you want to change"
        if change_whole_dic == False:
            if specific == None:
                for key in self.unique_experiments:
                    print(f"Current value for {key}: {self.unique_experiments[key]}")
                    new_value = input("Enter a new value or press any key to skip")
                    if len(new_value) > 1:
                        self.unique_experiments[key] = new_value
                    else:
                        pass
            else:
                new_value = input("Enter the new name for " + specific)
                self.unique_experiments[specific] = new_value
            self.sequencing_report["Experiment"] = self.sequencing_report["Experiment"].map(self.unique_experiments)
        else:
            with open("test_data" + "/experiment_names.pickle", "wb") as f:
                pickle.dump(change_whole_dic, f)

    def alignment_quality(self, log_transformation = False):
        """

        :param log_transformation: optional parameter. You can set it to True if you want to log transform your data
        :return: Creates a figure where you can see the Overall Reads per sample and the number of reads which could be aligned.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert log_transformation in [True, False], "You have to give True or False as input for the log transformation"
        self.fig.clear()
        self.ax = self.fig.gca()
        try:
            barplot(self.ax,
                    self.alignment_report,
                    self.sequencing_report,
                    self.font_settings,
                    self.legend_settings,
                    apply_log = log_transformation)
            self.plot_type = "single"
            self.ax = self.fig.gca()
            self.style = PlotStyle(self.ax, self.plot_type)
            figManager = plt.get_current_fig_manager()
            figManager.window.showMaximized()
        except:
            print("Currenlty you do not have loaded an alignment report.")

    def aa_distribution(self, sample, region, protein = True):
        """
        :param sample: The sample you would like to analyze
        :param region: the region you would like to analyze
        :param protein: Default True. If you would like to analyze nucleotide sequences, set it to False
        :return: Returns a plot which shows the amino acid distribution in the given sequence range.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert sample in self.experiments_list, "The provided sample name is not in your sequencing report. Please check the spelling or use the print_samples function to see the names of your samples"
        assert type(sample), "You have to give a string as input for the sample"
        assert type(region), "You have to give a list with the start and end position of the region you want to analyze. For instance: [3,7]"
        assert protein in [True, False], "You have to give True or False as input for the protein parameter"
        self.fig.clear()
        self.ax = self.fig.gca()
        stacked_aa_distr(self.ax,
                         self.sequencing_report,
                         sample,
                         region,
                         protein,
                         self.font_settings,
                         self.legend_settings)
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
    def usqPlot(self, samples):
        """
        :param samples: you insert a list which contains the sample names
        :return: USQ stands for unique sequences quality and the plot shows you the depth of unique sequences which can be used for evaluating your sequencing quality.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        incorrect_samples = [x for x in samples if x not in self.experiments_list]
        assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"

        if type(samples) != list:
            print("You have to give the sample names in the list, also if it is only one! A list is a container which is marked through:[] . Please try again.")
        else:
            self.fig.clear()
            self.ax = self.fig.gca()
            plot_USQ(fig = self.fig,
                    sequencing_report = self.sequencing_report,
                     samples = samples,
                     font_settings = self.font_settings,
                     legend_settings = self.legend_settings)
            self.plot_type = "single"
            self.ax = self.fig.gca()
            self.style = PlotStyle(self.ax, self.plot_type)
            figManager = plt.get_current_fig_manager()
            figManager.window.showMaximized()


    def logoPlot_single(self,
                        sample,
                        highlight_specific_pos = False,
                        highlight_pos_range = False,
                        chosen_seq_length = 16):
        """
        :param sample: insert the sample name
        :param highlight_specific_pos: optional. you can highlight a specific position
        :param chosen_seq_length: 16 per default. You always analyze online one sequence length! You can change it if you would like to.
        :return: A logo Plot which shows you the composition of aminoacids per position
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert sample in self.experiments_list, "The provided sample name is not in your sequencing report. Please check the spelling or use the print_samples function to see the names of your samples"
        assert type(sample) == str, "You have to give a string as input for the sample"
        if highlight_specific_pos != False:
            assert type(highlight_specific_pos) == int, "You have to give an integer as input for the specific position you want to highlight"
        if highlight_pos_range != False:
            assert type(highlight_pos_range) == list, "You have to give a list with the start and end position of the region you want to highlight. For instance: [3,7]"
        assert type(chosen_seq_length) == int, "You have to give an integer as input for the sequence length you want to analyze"
        self.fig.clear()
        self.ax = self.fig.gca()
        plot_logo_single(self.ax,
                         self.sequencing_report,
                         sample,
                         self.font_settings,
                         highlight_specific_pos,
                         highlight_pos_range,
                         chosen_seq_length)
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()


    def logoPlot_multi(self,
                num_cols,
                 samples = "all",
                 chosen_seq_length = 16,
                    ):
        """
        :param num_cols: number of columns you want to have in your figure.
        :param samples: You analyze all samples per default. If you want to analyze specific samples it has to be a list with the corresponding sample names
        :param chosen_seq_length: 16 per default. You always analyze online one sequence length! You can change it if you would like
        :return: Gives you in one figure one logoPlot per sample.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        if samples != "all":
            assert type(samples) == list, "You have to give a list with the samples you want to analyze"
            incorrect_samples = [x for x in samples if x not in self.experiments_list]
            assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        assert type(chosen_seq_length) == int, "You have to give an integer as input for the sequence length you want to analyze"
        self.fig.clear()
        plot_logo_multi(self.fig,
                  self.sequencing_report,
                  samples,
                  num_cols,
                  self.font_settings,
                  chosen_seq_length,
                    test_version = self.is_test
                  )

        self.plot_type = "multi"
        self.ax= self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        plt.tight_layout()

    def lengthDistribution_single(self, sample):
        """

        :param sample: name of the sample from which you would like to see the length distribution
        :return: Shows you the length Distribution of your sequences of the given sample
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert type(sample) == str, "You have to give a string as input for the sample"
        assert sample in self.experiments_list, "The provided sample name is not in your sequencing report. Please check the spelling or use the print_samples function to see the names of your samples"
        self.fig.clear()
        self.ax = self.fig.gca()
        length_distribution_single(self.fig,
                                   self.ax,
                                   self.sequencing_report,
                                   sample,
                                   self.font_settings)
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()

    def lengthDistribution_multi(self,num_cols, samples = "all"):
        """
        :param num_cols: number of columns you want to have in your figure.
        :param samples: You analyze ExpoSeq samples per default. If you want to analyze specific samples it has to be a list with the corresponding sample names
        :return: Outputs one figurewith one subplot per sample which shows you the distribution of sequence length
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        if samples != "all":
            assert type(samples) == list, "You have to give a list with the samples you want to analyze"
            incorrect_samples = [x for x in samples if x not in self.experiments_list]
            assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        self.fig.clear()
        length_distribution_multi(self.fig,
                            self.sequencing_report,
                            samples,
                            num_cols,
                            font_settings=self.font_settings,
                                test_version = self.is_test)
        self.plot_type = "multi"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        plt.tight_layout()


    def rel_seq_abundance(self, samples, max_levenshtein_distance = 0, length_filter = 0, batch = 3000):
        """

        :param samples: For a qualitative analysis choose samples from the same panning experiment. Input is a list
        :param max_levenshtein_distance: Default is 0. You can change it to see increased fraction with increased variability of certain sequences
        :param length_filter: Default is 0. You should change it if you change the levenshtein distance. Otherwise your results will be biased.
        :param batch: Default is 3000. The size of the sample which is chosen. The higher it is, the more computational intense.
        :return: Shows you a Bar Plot of the frequences of the most abundant sequences. You can introduce the levenshtein distance to see how the frequency changes with higher variability of the sequences.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        incorrect_samples = [x for x in samples if x not in self.experiments_list]
        assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        assert type(samples) == list, "You have to give a list with the samples you want to analyze"
        assert type(max_levenshtein_distance), "You have to give an integer as input for the maximum levenshtein distance"
        assert type(length_filter), "You have to give an integer as input for the length filter"
        assert type(batch) == int, "You have to give an integer as input for the batch size"
        self.fig.clear()
        self.ax = self.fig.gca()
        relative_sequence_abundance(self.ax,
                                    self.sequencing_report,
                                    samples,
                                    max_levenshtein_distance,
                                    length_filter,
                                    batch,
                                    self.font_settings,
                                    self.legend_settings)
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()


    def basic_cluster(self, sample,max_ld = 1, min_ld = 0, second_figure = False):
        """
        :param sample: type in a sample name you want to analyze
        :return:
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert sample in self.experiments_list, "The provided sample name is not in your sequencing report. Please check the spelling or use the print_samples function to see the names of your samples"
        assert type(sample) == str, "You have to give a string as input for the sample"
        assert type(max_ld) == int, "You have to give an integer as input for the maximum levenshtein distance"
        assert type(min_ld) == int, "You have to give an integer as input for the minimum levenshtein distance"
        assert type(second_figure) == bool, "You have to give True or False as input for the second figure"
        self.fig.clear()
        self.ax = self.fig.gca()
        fig2 = clusterSeq(
                   self.ax,
                   self.sequencing_report,
                   sample,
                    max_ld,
                    min_ld,
                   self.batch_size,
                    self.font_settings,
                    second_figure)

        self.ax = self.fig.gca()
        self.plot_type = "single"
        self.style = PlotStyle(self.ax,
                               self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        if second_figure == True:
            print("close the second window before you continue")



    def cluster_one_AG(self, antigen,max_ld = 1, min_ld = 0, specific_experiments=False):
        """

        :param antigen: is the name of the antigen you would like to analyze
        :param max_ld: optional Parameter where its default is 1. Is the maximum Levenshtein distance you allow per cluster
        :param min_ld: optional Parameter where its default is 0. Is the minimum Levenshtein distance between sequences you allow
        :param specific_experiments: optional Parameter. You can give the names of specific samples in a list if you want
        :return: Creates a figure where sequences are clustered based on Levenshtein distance. Additionally the binding data of the sequences against a specific antigen is given.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert self.binding_data is not None, "You have not given binding data. You can add it with the add_binding_data function"
        assert type(antigen) == str, "You have to give a string as input for the antigen"
        assert type(max_ld) == int, "You have to give an integer as input for the maximum levenshtein distance"
        assert type(min_ld) == int, "You have to give an integer as input for the minimum levenshtein distance"
        assert type(specific_experiments) == list, "You have to give a list with the samples you want to analyze"
        self.fig.clear()
        cluster_single_AG(self.fig,
                          self.sequencing_report,
                          antigen,
                          self.binding_data,
                          max_ld,
                          min_ld,
                          self.batch_size,
                          specific_experiments,
                          )
        self.plot_type = "multi"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()

    def tsne_cluster_AG(self, sample, antigen,antigen_names = True, pca_components = 70, perplexity = 25, iterations_tsne = 2500):
        """
        :param sample: the sample you would like to analyze
        :param antigen: the toxins you would like to cluster
        :param antigen_names: Default is True. Prints the name of the toxin for the corresponding embedded sequence in the plot
        :param pca_components: optional. Default is 70
        :param perplexity: optional. Default 25
        :param iterations_tsne: optional. Default is 2500
        :return: It first embeds the sequences in a vector space and then clusters them with PCA and TSNE. The sequences with the binding data are processed with the input sequences, to enable the plotting of the binding data.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        assert self.binding_data is not None, "You have not given binding data. You can add it with the add_binding_data function"
        assert type(sample) == str, "You have to give a string as input for the sample"
        assert sample in self.experiments_list, "The provided sample name is not in your sequencing report. Please check the spelling or use the print_samples function to see the names of your samples"
        assert antigen_names in [True, False], "You have to give True or False as input for the antigen_names parameter"
        assert type(pca_components) == int, "You have to give an integer as input for the pca_components"
        assert type(perplexity) == int, "You have to give an integer as input for the perplexity"
        assert type(iterations_tsne) == int, "You have to give an integer as input for the iterations_tsne"
        assert type(antigen) == list, "You have to give a list with the antigens you want to analyze"
        
        self.fig.clear()
        #self.ax = self.fig.gca()
        tsne_results = cluster_toxins_tsne(self.fig,
                            self.sequencing_report,
                            sample,
                            antigen,
                            self.binding_data,
                            antigen_names,
                            pca_components,
                            perplexity,
                            iterations_tsne,
                            self.font_settings,
                            self.colorbar_settings,
                            extra_figure = False)
        self.plot_type = "multi"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        save_embedding = input("Do you want to save the corresponding data as csv? (Y/n)?")
        if save_embedding.lower() in ["Y", "y"]:
            name_csv = input("How do you want to call the file")
            path_save_embedding = os.path.join(save_embedding, name_csv + ".csv" )
            tsne_results.to_csv(path_save_embedding, index=False)

    def embedding_tsne(self,
                       samples,
                       strands = True,
                       pca_components = 80,
                       perplexity = 30,
                       iterations_tsne = 2500):
        """
        :param samples: the samples you would like to compare towards their sequences
        :param strands: Default is True. It means that you will plot a batch of the strands in your plot
        :param pca_components: Default is 80. Has to be applied for better accuracy of t-SNE. You can indirectly change the described variance with this.
        :param perplexity: Default is 30. It roughly determines the number of nearest neighbors that are considered in the embedding. A higher perplexity value results in a more global structure in the low-dimensional embedding, while a lower perplexity value emphasizes local structure. The optimal perplexity value for a given dataset depends on the dataset's intrinsic dimensionality, and it is usually determined by trial and err
        :param iterations_tsne: Default is 2500. number of times that the algorithm will repeat the optimization process for reducing the cost function. The optimization process aims to minimize the difference between the high-dimensional and low-dimensional representations of the data. More iterations result in a more optimized low-dimensional representation, but also increases the computational cost.
        :return: Returns a plot where the sequences of the input samples are transformed in a vector space. Dimension reduction such as PCA and following t-SNE is used to plot it on a two dimensional space. The different colors indicate the different samples.
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        incorrect_samples = [x for x in samples if x not in self.experiments_list]
        assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        assert type(samples) == list, "You have to give a list with the samples you want to analyze"
        assert type(strands) == bool, "You have to give True or False as input for the strands parameter"
        assert type(pca_components) == int, "You have to give an integer as input for the pca_components"
        assert type(perplexity) == int, "You have to give an integer as input for the perplexity"
        assert type(iterations_tsne) == int, "You have to give an integer as input for the iterations_tsne"
        self.fig.clear()
        self.ax = self.fig.gca()
        show_difference(self.sequencing_report,
                        samples,
                        strands,
                        self.batch_size,
                        pca_components,
                        perplexity,
                        iterations_tsne,
                        self.ax,
                        self.legend_settings,
                        self.font_settings)
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
    def morosita_horn(self, specific_experiments = False):
        """
        :param specific_experiments: you can give a list with specific experiments
        :return: Returns a matrix of the identity between your samples based on the Morosita Horn Index
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        if specific_experiments != False:
            assert type(specific_experiments) == list, "You have to give a list with the samples you want to analyze"
            incorrect_samples = [x for x in specific_experiments if x not in self.experiments_list]
            assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        self.fig.clear()
        self.ax = self.fig.gca()
        plot_heatmap(self.sequencing_report,
                     True,
                     "morosita_horn",
                     self.ax,
                     self.colorbar_settings,
                     self.font_settings,
                     specific_experiments = specific_experiments,
                     )
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        return self.fig

            
    def jaccard(self, specific_experiments = False):
        """
        :param specific_experiments: give a list with specific samples you would like to analyze
        :return: Returns a matrix of the identity between your samples based on the Jaccard Index
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        if specific_experiments != False:
            assert type(specific_experiments) == list, "You have to give a list with the samples you want to analyze"
            incorrect_samples = [x for x in specific_experiments if x not in self.experiments_list]
            assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        self.fig.clear()
        self.ax = self.fig.gca()
        plot_heatmap(self.sequencing_report,
                     True,
                     "jaccard",
                     self.ax,
                     self.colorbar_settings,
                     self.font_settings,
                     specific_experiments = specific_experiments,
                     )
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()

    def sorensen(self, specific_experiments = False):
        """
        :param specific_samples: give a list with specific samples you would like to analyze
        :return: Returns a matrix of the identity between your samples based on the Sorensen Dice Index
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        if specific_experiments != False:
            assert type(specific_experiments) == list, "You have to give a list with the samples you want to analyze"
            incorrect_samples = [x for x in specific_experiments if x not in self.experiments_list]
            assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        self.fig.clear()
        self.ax = self.fig.gca()
        plot_heatmap(self.sequencing_report,
                     True,
                     "sorensen",
                     self.ax,
                     self.colorbar_settings,
                     self.font_settings,
                     specific_experiments = specific_experiments,
                     )
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()

    def relative(self, specific_experiments = False):
        """
        :param specific_samples: give a list with specific samples you would like to analyze
        :return: Returns a matrix of the identity between your samples based on the proportion of identical sequences
        """
        if not plt.fignum_exists(1):
            self.fig = plt.figure(1)
            print("Please do not close the window for the figure while the plot is loading")
        if specific_experiments != False:
            assert type(specific_experiments) == list, "You have to give a list with the samples you want to analyze"
            incorrect_samples = [x for x in specific_experiments if x not in self.experiments_list]
            assert not incorrect_samples, f"The following sample(s) are not in your sequencing report: {', '.join(incorrect_samples)}. Please check the spelling or use the print_samples function to see the names of your samples"
        self.fig.clear()
        self.ax = self.fig.gca()
        plot_heatmap(self.sequencing_report,
                     True,
                     "relative",
                     self.ax,
                     self.colorbar_settings,
                     self.font_settings,
                     specific_experiments = specific_experiments,
                     )
        self.plot_type = "single"
        self.ax = self.fig.gca()
        self.style = PlotStyle(self.ax, self.plot_type)
        figManager = plt.get_current_fig_manager()
        figManager.window.showMaximized()
        


















from ExpoSeq.tidy_data.tidy_rel_sequence_abun import cleaning
def relative_sequence_abundance(ax, sequencing_report, samples,max_levenshtein_distance,length_filter,batch, font_settings, legend_settings):
    all_samples = cleaning(sequencing_report,
                            max_levenshtein_distance,
                            samples,
                            length_filter,
                            batch)
    x_vals = all_samples.index.to_list()
    bar_width = 0.25
    all_samples_columns = all_samples.columns.to_list()
    for i in range(all_samples.shape[1]):
        y = all_samples.iloc[:, i]
        x_pos = [j + i * bar_width for j in range(len(x_vals))]
        ax.bar(x_pos,
               y,
               width=0.25,
               align='center',
               label=all_samples_columns[i])
    ax.set_xticks(range(len(x_vals)),
                  x_vals,
                  rotation = 45,
                  ha = "right")
    ax.set_ylabel("Frequency in samples with Levenshtein Distance " + str(max_levenshtein_distance),
                  **font_settings)
    ax.set_xlabel("Sequences",
                  **font_settings)
    ax.legend(**legend_settings)
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    ax.set_title("Sequence Abundance of given samples",
                 pad = 12,
                 **font_settings)
    font_settings["fontsize"] = original_fontsize



import pandas as pd
import os

try:
    import tkinter as tk
    from tkinter import filedialog
except:
    pass

def collect_binding_data(binding_data = None):
    if binding_data == None:
        binding_data = pd.DataFrame([])
    else:
        pass
    while True:
        # prompt the user to add a file
        print("add your excel sheet with the binding data with the file chooser")

        try:
            binding_file = filedialog.askopenfilename()
        except:
            while True:
                binding_file = input("copy and paste the path to your binding report")
                if os.path.isfile(os.path.abspath(binding_file)):
                    break
                else:
                    print("Please enter a valid filepath. ")
        binding_data = pd.concat([binding_data, binding_file])
        response = input("Do you want to continue adding files? (Y/n) ")
        if response.lower() == "n":
            break
    return binding_data


import numpy as np
import pandas as pd
from .bool_sequences_matrix import find_seq_matches


def heatmap_sorensen(sequencing_report, protein, specific_experiments):
    heatmap_axis, unique_sequences, unique_experiments = find_seq_matches(sequencing_report,
                                                                          protein,
                                                                          specific_experiments)
    heatmap_absolute_sorensen = np.zeros([heatmap_axis, heatmap_axis])
    for index_sample_one in range(heatmap_axis):
        d1 = unique_sequences.iloc[:, index_sample_one]
        sequencing_report[sequencing_report["Experiment"] == unique_experiments[index_sample_one]].shape[0]
        for index_sample_two in range(heatmap_axis):
            sequencing_report[sequencing_report["Experiment"] == unique_experiments[index_sample_two]].shape[0]
            d2 = unique_sequences.iloc[:, index_sample_two]
            lib_matches = d1 * d2
            lib_matches = lib_matches.sum()
            b = d1.sum() - lib_matches
            c = d2.sum() - lib_matches
            heatmap_absolute_sorensen[index_sample_one, index_sample_two] = 2 * lib_matches / (2 * lib_matches + b + c)
            heatmap_absolute_sorensen[index_sample_two, index_sample_one] = 2 * lib_matches / (2 * lib_matches + b + c)
    matrix = pd.DataFrame(heatmap_absolute_sorensen,
                          index = list(unique_experiments),
                          columns = list(unique_experiments))
    return matrix, unique_sequences, unique_experiments


import matplotlib.pyplot as plt
from numpy import unique
import networkx
import math
import pandas as pd
from ExpoSeq.tidy_data.clustering import cleaning


def clusterSeq(ax, sequencing_report, sample,max_ld, min_ld, batch_size, font_settings, second_figure):
    sample_report = sequencing_report[sequencing_report["Experiment"] == sample] ## insert test if sample not found
    sample_report = sample_report.head(batch_size)
    G, degree_sequence = cleaning(sample_report, max_ld, min_ld)
    # Create a gridspec for adding subplots of different sizes
    clone_counts = sample_report["readCount"]
    G_deg = G.degree()
    to_remove = [n for (n, deg) in G_deg if deg == 0]
    G.remove_nodes_from(to_remove)
    nodesize = []
    for i in sample_report.index:
        for g in G:
            if i == g:
                node = math.sqrt(clone_counts.iloc[i])
                nodesize.append(node)
    for y, n in enumerate(nodesize):
        if n > 0 and n < 15:
            nodesize[y] = 15
        if n == 0:
            nodesize[y] = 50
    networkx.draw_networkx(G, arrows = True, with_labels = False,ax = ax)
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    ax.set_title("Connected components of " + sample, **font_settings)
    font_settings["fontsize"] = original_fontsize
    ax.set_axis_off()
    if second_figure == True:
        fig2 = plt.figure()
        ax2 = fig2.gca()
        ax2.bar(*unique(degree_sequence, return_counts=True))
        original_fontsize = font_settings["fontsize"]
        font_settings["fontsize"] = 22
        ax2.set_title(sample + ' Histogram', **font_settings)
        font_settings["fontsize"] = original_fontsize
        ax2.set_xlabel("Degree", **font_settings)
        ax2.set_ylabel("# of Nodes", font_settings)
        return fig2
    else:
        return

def cluster_single_AG(fig, sequencing_report, antigen, binding_data, max_ld, min_ld, batch_size, specific_experiments = False, ):
    report_batch = sequencing_report.groupby("Experiment").head(batch_size)
    if specific_experiments != False:
        report_batch = report_batch[report_batch['Experiment'].isin(specific_experiments)]
    else:
        pass
    experiments = report_batch["Experiment"].unique()
    Tot = experiments.shape[0]
    Cols = int(input("How many Columns for the Plots do you want?"))
    Rows = Tot // Cols
    if Tot % Cols != 0:
        Rows += 1
    # Create a Position index
    Position = range(1, Tot + 1)
    plot_number = 0

    for experiment in experiments:
        ax = fig.add_subplot(Rows, Cols, Position[plot_number])
        batch = report_batch[report_batch["Experiment"] == experiment]
        mix = pd.concat([batch, binding_data])
        mix = mix.fillna(0)
        mix = mix.reset_index()
        G, degree_sequence = cleaning(mix, max_ld, min_ld)
        G_deg = G.degree()
        to_remove = [n for (n, deg) in G_deg if deg == 0]
        G.remove_nodes_from(to_remove)
        nodesize = []
        clone_counts = mix["readCount"]
        color_values = []
        for i in mix.index:
            for g in G:
                if i == g:
                    node = math.sqrt(clone_counts.iloc[i])
                    nodesize.append(node)
                    color_values.append(mix[antigen].iloc[i])
        for y, n in enumerate(nodesize):
            if n>0 and n < 15:
                nodesize[y] = 15
            if n == 0:
                nodesize[y] = 50
        cm = plt.get_cmap("Blues")
        vmin = min(color_values)
        vmax = max(color_values)
        network_plot = networkx.draw_networkx(G,
                                              node_size = nodesize,
                                              node_color = color_values,
                                              cmap = cm,
                                              with_labels = False,
                                              arrows = True,
                                              edgecolors = "Black",
                                              ax = ax)
     #   net = networkx.draw_networkx(G, node_size=nodesize, node_color = color_values, cmap = cm,with_labels=False, edgecolors = "Black", ax = ax)
       # network_plot = networkx.draw_networkx_nodes(G, pos=networkx.spring_layout(G), node_size=nodesize, node_color = color_values, cmap = cm,edgecolors = "Black", vmin = vmin, vmax = vmax)
        ax.set_title(experiment)
     #   sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=vmin, vmax=vmax))
      #  cbar = plt.colorbar(sm)
        plot_number = plot_number+1
    fig.colorbar(network_plot,
                 ax=fig.axes)

antigens = ['Ecarpholin_S_DDEL_5ug/mL', 'Myotoxin_II_DDEL_5ug/mL',
           'PLA2_n.naja(Uniprot:P15445)_DDEL_5ug/mL', 'Ecarpholin_S_CDEL_100nM',
           'Myotoxin_II_CDEL_100nM',]



### is not in use
def cluster_antigens(sequencing_report, sample, antigens, batch_size):
    report_batch = sequencing_report.groupby("Experiment").head(batch_size)
    Tot = len(antigens)
    Cols = int(input("How many Columns for the Plots do you want?"))
    # Compute Rows required
    Rows = Tot // Cols
    #     EDIT for correct number of rows:
    #     If one additional row is necessary -> add one:
    if Tot % Cols != 0:
        Rows += 1
    # Create a Position index
    Position = range(1, Tot + 1)
    plot_number = 0
    fig = plt.figure(1)
    max_colorbar = 0
    batch = report_batch[report_batch["Experiment"] == sample]
    mix = pd.concat([batch, delfia_samples])
    mix = mix.fillna(0)
    mix = mix.reset_index()
    mix = mix.drop("index", axis=1)
    G, degree_sequence = cleaning(mix)
    G_deg = G.degree()
    to_remove = [n for (n, deg) in G_deg if deg == 0]
    G.remove_nodes_from(to_remove)
    for antigen in antigens:
        ax = fig.add_subplot(Rows, Cols, Position[plot_number])
        nodesize = []
        clone_counts = mix.readCount
        color_values = []

        for i in range(mix.shape[0]):
            for g in G:
                if i == g:
                    node = math.sqrt(clone_counts.iloc[i])
                    nodesize.append(node)
                    color_values.append(mix[antigen].iloc[i])
        for y, n in enumerate(nodesize):
            if n>0 and n<15:
                nodesize[y] = 15
            if n == 0:
                nodesize[y] = 50

        colormap = plt.get_cmap("Blues")
        vmin = min(color_values)
        vmax = max(color_values)
        if vmax > max_colorbar:
            max_colorbar = vmax
        #   net = networkx.draw_networkx(G, node_size=nodesize, node_color = color_values, cmap = cm,with_labels=False, edgecolors = "Black", ax = ax)
        net = networkx.draw_networkx_nodes(G,
                                           pos=networkx.spring_layout(G),
                                           node_size=nodesize,
                                           node_color=color_values,
                                           cmap=colormap,
                                     edgecolors="Black")
        ax.set_title(antigen)
        #   sm = plt.cm.ScalarMappable(cmap=cmap, norm=plt.Normalize(vmin=0, vmax=max_colorbar))
        #  cbar = plt.colorbar(sm)
        plot_number = plot_number + 1
    fig.suptitle("Clustering for sample " + sample)
#    sm = plt.cm.ScalarMappable(cmap=colormap,
 #                              norm=plt.Normalize(vmin = vmin, vmax=max_colorbar))

    #fig.colorbar(ax, col_ax)


















from ExpoSeq.augment_data.mixcr_nils import process_mixcr
from ast import literal_eval
import os
from ExpoSeq.augment_data.loop_collect_reports import load_mixed_files, load_alignment_reports
import pandas as pd
from glob import glob
from ExpoSeq.augment_data.check_reports import check_completeness
import pickle
import pkg_resources
from ExpoSeq.augment_data.trimming import trimming
try:
    import tkinter as tk
    from tkinter import filedialog
except:
    pass
import random
import string
import shutil

def get_random_string(length):
    letters = string.ascii_letters  # this will get all the uppercase and lowercase letters
    result_str = ''.join(random.choice(letters) for i in range(length))
    return result_str

def pull_seq_align_repo(testing, testing_value):
    if not testing:
        try:
            print("choose the directory where you store your alignment reports and your sequencing report")
            filenames_dir = filedialog.askdirectory()
            assert os.path.isdir(filenames_dir), "Please enter a valid directory."
            filenames = glob(filenames_dir + "//*")
           
        except:
            filenames_dir = input(
                "Enter the directory where you store the alignment reports and your sequencing report manually. You should have one file for the sequencing report and multiple files for the alignment reports.")
            while True:
                if os.path.isdir(filenames_dir):
                    break
                else:
                    print("Please enter a valid directory.")
                    filenames_dir = input(
                        "Enter the directory where you store the alignment reports and your sequencing report manually. You should have one file for the sequencing report and multiple files for the alignment reports.")
            filenames_dir = os.path.abspath(filenames_dir + "//*")
            filenames = glob(filenames_dir)
    else:
        module_dir = os.path.abspath("")
        filenames_dir = os.path.join(module_dir,
                                    "tests",
                                    "test_files")
        filenames = glob(filenames_dir + "//*")
    for i in filenames:
        if os.path.isdir(i):
            if "alignment_reports" in i:
                alignment_repos = glob(i)
                filenames.extend(alignment_repos)
        sequencing_report, all_alignment_reports = load_mixed_files(filenames, testing, testing_value)
    return sequencing_report, all_alignment_reports

def check_experiment(module_dir, testing):
    if not testing:
        while True:
            experiment = input("How do you want to call your new experiment?")
            if os.path.isdir(os.path.join(module_dir, "my_experiments", experiment)) == True:
                replace = input("The given directory already exists. Do you want to replace it? (Y/n)")
                if replace in ["Y", "y", "n", "N"]:
                    if replace in ["Y", "y"]:
                        shutil.rmtree(os.path.join(module_dir, "my_experiments", experiment))
                    break
                else:
                    print("Please enter another name.")
            else:
                break


    else:
        experiment = get_random_string(10)
    os.mkdir(os.path.join(module_dir,
                    "my_experiments",
                    experiment))
    return experiment

def method_one(experiment, repo_path, module_dir, testing, paired_end_test = "n"):
    if repo_path == "":
        repo_path = os.path.join(module_dir, "my_experiments", experiment, "sequencing_report.csv")
    if not testing:
        use_method = input(
            "Per default you will align your data with the following method: milab-human-tcr-dna-multiplex-cdr3 . Press enter if you want to continue. Otherwise type in the method of your choice which. It has to be the exact same string which is given on the Mixcr documentation.")
    else:
        use_method = ""
    if use_method == "":
        method = "milab-human-tcr-dna-multiplex-cdr3"
    else:
        method = use_method
    if not testing:
        paired_end = input("Do you want to analyze paired_end_sequencing data? (Y/n)")
       
    else:
        paired_end = 'n'
    while True:
        if paired_end in ["Y", "y", "n", "N"]:
            break
        else:
            print("Please enter a correct value.")
            paired_end = input("Do you want to analyze paired_end_sequencing data? (Y/n)")
    if paired_end.lower() in ["Y", "y"]:
        paired_end = True
    else:
        paired_end = False

    process_mixcr(experiment,
                  method=method,
                  testing = testing,
                  paired_end_sequencing=paired_end)

    with open(repo_path, "rb") as f:
        sequencing_report = pd.read_table(f, sep=",")
    try:
        align_repo_path = os.path.join(module_dir,
                                       "my_experiemnts",
                                       experiment,
                                       "alignment_reports",
                                       "*")
        alignment_reports = glob(align_repo_path)
        all_alignment_reports = load_alignment_reports(alignment_reports)
    except:
        all_alignment_reports = pd.DataFrame([])
        print(
            "No alignment reports could be found in " + experiment + ". You will continue without being able to analyze the Alignment Quality.")
    return sequencing_report, all_alignment_reports
def method_two(module_dir, experiment, testing = False, testing_value = None):

    print("Choose the folder which contains the sequencing_report and the alignment reports.")

    sequencing_report, all_alignment_reports = pull_seq_align_repo(testing, testing_value)

    while True:
        if sequencing_report.shape[0] == 0:
            print(
                "Something went wrong. Unfortunately you could not collect the data in the sequencing report.")
            sequencing_report, all_alignment_reports = pull_seq_align_repo(testing, testing_value)
        else:
            break
        #  sys.exit()
    if not testing:
        while True:
            trim_data = input("Do you need to trim your data? (Y/n)")
            if trim_data in ["Y", "y", "n", "N"]:
                break
            else:
                print("Please enter a correct value.")
    else: trim_data = "Y"
    if trim_data.lower() in ["Y", "y"]:
        sequencing_report = trimming(sequencing_report,
                                     divisible_by=3,  # might be possible that columns are different
                                     min_count=1,
                                     new_fraction="cloneFraction")
    else:
        pass
    seq_report_dir = os.path.join(module_dir,
                                  "my_experiments",
                                  experiment,
                                  "sequencing_report.csv")
    sequencing_report.to_csv(seq_report_dir)
    unique_experiments = sequencing_report["Experiment"].unique()
    experiment_dic = {item: item for item in list(unique_experiments)}
    exp_names_dir = os.path.join(module_dir,
                                 "my_experiments",
                                 experiment,
                                 "experiment_names.pickle")
    with open(exp_names_dir, "wb") as f:
        pickle.dump(experiment_dic, f)

    if all_alignment_reports.shape[0] == 0:
        print(
            "No Alignment Reports were uploaded. You will continue the analysis without being able to analyze the Alignment Quality.")
    else:
        try:
            all_alignment_reports = check_completeness(all_alignment_reports, sequencing_report)
        except:
            pass
    exp_name_path = os.path.join(module_dir,
                                 "my_experiments",
                                 experiment,
                                 "experiment_names.pickle")

    unique_experiments = sequencing_report["Experiment"].unique()
    experiment_dic = {item: item for item in list(unique_experiments)}

    with open(exp_name_path, "wb") as f:
        pickle.dump(experiment_dic, f)
    return sequencing_report, all_alignment_reports

def method_three(module_dir, experiment, testing ):
    if not testing:
        try:
            print("choose the file where you store your sequencing report")
            while True:
                f = filedialog.askopenfilename()
                if os.path.isfile(f):
                    break
                else:
                    print("Please enter a valid file path")
        except:
            f = input("give the path to your file which is the sequencing report")
            f = os.path.abspath(f)
            while True:
                if os.path.isfile(f):
                    break
                else:
                    print("Please enter a valid file path")
    else:

        f = os.path.join(module_dir,
                        "my_experiments",
                        "test_directory",
                        "sequencing_report.csv")
        experiment = "test_directory"
    while True:
        sequencing_report = pd.read_table(f, sep=",")
        if os.path.isfile(f) :
            sequencing_report = pd.read_table(f, sep=",")
            if "Experiment" in sequencing_report:
                exp_name_path = os.path.join(module_dir,
                                            "my_experiments",
                                            experiment,
                                            "experiment_names.pickle")

                unique_experiments = sequencing_report["Experiment"].unique()
                experiment_dic = {item: item for item in list(unique_experiments)}
                with open(exp_name_path, "wb") as f:
                    pickle.dump(experiment_dic, f)
                break
            else:
                print("Sorry, but we could not find the column Experiment in your sequencing report. Please check if you have this")
        else:
            print("Sorry, but the filepath you entered is invalid.")
    all_alignment_reports = None
    seq_report_dir = os.path.join(module_dir,
                                  "my_experiments",
                                  experiment,
                                  "sequencing_report.csv")
    sequencing_report.to_csv(seq_report_dir)
    return sequencing_report,all_alignment_reports, experiment

def check_last_exp(pkg_path, module_dir, testing):
    if not testing:
        glob_vars = os.path.join(pkg_path,
                                "settings",
                                "global_vars.txt")
        with open(glob_vars, "r") as f:
            data = f.read()
        data = literal_eval(data)

        last_experiment = data["last_experiment"]
        if last_experiment == "":
            print("You have no experiments in your folder.")
            repo_path = ""
        else:
            repo_path = os.path.join(module_dir,
                                   "my_experiments",
                                    last_experiment,
                                    "sequencing_report.csv")
    else:
        repo_path = os.path.join(module_dir,
                                "my_experiments",
                                "test_directory",
                                "sequencing_report.csv")
        last_experiment = "test_directory"
    return repo_path, last_experiment
def get_continue_analysis_input(last_experiment, testing=False, testing_value = None):
    if not testing:
        continue_input = input("Do you want to continue to analyze with " + last_experiment + "? Y/n")
    else:
        continue_input = testing_value
    return continue_input

def get_next_step_input(testing=False, testing_value = None):
    if not testing:
        next_step = input("If you want to upload a new experiment press 1. If you want to choose another experiment press 2. ")
    else:
        next_step = testing_value
    return next_step

def get_choose_method_input(testing, testing_value = None):
    if not testing:
        processing = input("If you want to process your fastq files with mixcr press 1. If you want to upload already processed txt files with unique clones, press 2. If you want to upload an already processed sequencing report in table format, press 3.")
    else:
        processing = testing_value
    return processing

def get_experiment_name_input(testing = False):
    if not testing:
        experiment_name = input("Enter the name of the experiment you want to analyze")
    else:
        experiment_name = get_random_string(10)
    return experiment_name 

def upload_new_experiment(module_dir, repo_path, testing, testing_value, paired_end_test, experiment_column):
    experiment = check_experiment(module_dir, testing)
    choose_method = get_choose_method_input(testing, testing_value)
    
    while True:
        if choose_method in ["1", "2", "3"]:
            if choose_method == "1":
            
                sequencing_report, all_alignment_reports = method_one(experiment, repo_path,module_dir,testing, paired_end_test )
            elif choose_method == "2":
                sequencing_report, all_alignment_reports = method_two(module_dir, experiment,testing, experiment_column)
            elif choose_method == "3":
                sequencing_report, all_alignment_reports, experiment = method_three(module_dir, experiment, testing)
            break
        else:
            print("Please enter a correct value.")

        

    return sequencing_report, all_alignment_reports, experiment

def choose_existing_experiment(module_dir, testing):
    if not testing:
        while True:
            user_input = get_experiment_name_input()
            spec_exp_name_path = os.path.join(module_dir, "my_experiments", user_input)
            if os.path.isdir(spec_exp_name_path):
                break
            else:
                print("The experiment name does not exist in my_experiments. Please enter the correct name")
    else:
        user_input = "test_directory"
    return user_input  # experiment



def retrieve_reports(module_dir, experiment, testing):
    if not testing:
        seq_report_path = os.path.join(module_dir, "my_experiments", experiment, "sequencing_report.csv")
        align_repo_path = os.path.join(module_dir,
                            "my_experiments",
                            experiment,
                            "alignment_reports", "*")
    else:
        seq_report_path = os.path.join(module_dir, "my_experiments", "test_directory", "sequencing_report.csv")
        align_repo_path = os.path.join(module_dir, "tests", "test_directory", "all_alignment_reports.pickle")
    with open(seq_report_path, "rb") as f:
        sequencing_report = pd.read_table(f, sep=",")
    try:
        align_repo_path = os.path.join(module_dir,
                                    "my_experiments",
                                    experiment,
                                    "alignment_reports", "*")
        alignment_reports = glob(align_repo_path)
        all_alignment_reports = load_alignment_reports(alignment_reports)
    except:
        all_alignment_reports = pd.DataFrame([])
        print("No alignment reports could be found in " + experiment + ". You will continue without being able to analyze the Alignment Quality.")

    return sequencing_report, all_alignment_reports, experiment

def process_data_with_last_experiment(module_dir, last_experiment, testing):
    experiment = last_experiment
    sequencing_report, all_alignment_reports, experiment = retrieve_reports(module_dir, experiment, testing)
    return sequencing_report, all_alignment_reports, experiment

def upload(testing=False, continue_analysis = "n", upload_type = "2", choose_exp = '1', paired_end_test = 'n', experiment_column = '1'):
   # module_dir = os.path.abspath("")
    module_dir = os.getcwd()
    pkg_path = pkg_resources.resource_filename("ExpoSeq", "")
    repo_path, last_experiment = check_last_exp(pkg_path, module_dir, testing)

    if os.path.isfile(repo_path):
        continue_analysis = get_continue_analysis_input(last_experiment, testing,continue_analysis)
        if continue_analysis.lower() in ["n", "no"]:
            next_step = get_next_step_input(testing, choose_exp)
            if next_step == "1":
                sequencing_report, all_alignment_reports, experiment = upload_new_experiment(module_dir,repo_path, testing, upload_type, paired_end_test, experiment_column)
            elif next_step == "2":
                experiment = choose_existing_experiment(module_dir, testing)
                sequencing_report, all_alignment_reports, experiment = retrieve_reports(module_dir, experiment, testing)
            else:
                raise ValueError("Invalid option")
        else:
            sequencing_report, all_alignment_reports, experiment = process_data_with_last_experiment(module_dir, last_experiment, testing)
    else:
        sequencing_report, all_alignment_reports, experiment = upload_new_experiment(module_dir, repo_path, testing, upload_type, paired_end_test, experiment_column)
    if testing and experiment != "test_directory":
        shutil.rmtree(os.path.join(module_dir, "my_experiments", experiment))
    return sequencing_report, all_alignment_reports, experiment


import os
from ast import literal_eval

import pkg_resources
try:
    import tkinter as tk
    from tkinter import filedialog
except:
    pass
class Change_save_settings():
    def __init__(self):
        self.pkg_path = pkg_resources.resource_filename("ExpoSeq", "")

        self.save_settings_path = os.path.join(self.pkg_path,
                                          "settings",
                                               "save_settings.txt")
        with open(self.save_settings_path, "r") as f:
            save_settings = f.read()
        self.save_settings = literal_eval(save_settings)
    def change_dpi(self, dpi):
        self.save_settings["dpi"] = dpi
        with open(self.save_settings_path, "w") as f:
            f.write(str(self.save_settings))
    def change_save_path(self, path = None):
        if path == None:
            try:
                self.save_settings["fname"] = filedialog.askopenfilename()
            except:
                while True:
                    save_dir = input("copy and paste the path to the directory where you want to save your plots")
                    if os.path.isdir(os.path.abspath(save_dir)):
                        self.save_settings["fname"] = save_dir
                        break
                    else:
                        print("Try again and input a valid directory")
        else:
            self.save_settings["fname"] = path
            print("The new path you safe your results in is: " + path)
        with open(self.save_settings_path, "w") as f:
            f.write(str(self.save_settings))

    def change_format(self, format):
        self.save_settings["format"] = format
        with open(self.save_settings_path, "w") as f:
            f.write(str(self.save_settings))

    def print_save_path(self):
        print(self.save_settings["fname"])




import numpy as np
from .bool_sequences_matrix import find_seq_matches
import pandas as pd

def heatmap_share(sequencing_report, protein, specific_experiments):
    heatmap_axis, unique_sequences, unique_experiments = find_seq_matches(sequencing_report,
                                                                          protein,
                                                                          specific_experiments)
    heatmap_absolute = np.zeros([heatmap_axis, heatmap_axis])


    for index_sample_one in range(heatmap_axis):
        d1 = unique_sequences.iloc[:, index_sample_one]
        shape_first_sample = sequencing_report[sequencing_report["Experiment"] == unique_experiments[index_sample_one]].shape[0]
        for index_sample_two in range(heatmap_axis):
            shape_second_sample = \
                sequencing_report[sequencing_report["Experiment"] == unique_experiments[index_sample_two]].shape[0]

            d2 = unique_sequences.iloc[:, index_sample_two]
            inner_join = np.where(np.logical_and(d1 == 1, d2 == 1))
            identity_both = inner_join[0].shape[0]
            lib_matches = d1 * d2
            lib_matches = lib_matches.sum()
            a = lib_matches
            b = d1.sum() - lib_matches
            c = d2.sum() - lib_matches
            heatmap_absolute[index_sample_one, index_sample_two] = float(lib_matches / ((shape_second_sample + shape_first_sample)/2))
            heatmap_absolute[index_sample_two, index_sample_one] = float(lib_matches / ((shape_second_sample + shape_first_sample)/2))
    matrix = pd.DataFrame(heatmap_absolute,
                          index=list(unique_experiments),
                          columns=list(unique_experiments))
    return matrix, unique_sequences, unique_experiments


from ExpoSeq.tidy_data.tidy_USQ_plot_ import cleaning_data
import matplotlib.pyplot as plt
import random
from collections import Counter
import warnings
import matplotlib
matplotlib.use('Qt5Agg')
def plot_USQ(fig, sequencing_report, samples, font_settings, legend_settings):
    sub_table = cleaning_data(sequencing_report = sequencing_report,
                              samples = samples)
    x_axis = []
    y_axis = []
    for sample in range(len(sub_table)):
        sequences = list(sub_table.iloc[sample]["nSeqCDR3"])
        fraction = list(sub_table.iloc[sample]["clonesFraction"])
        counter = 0
        temp_list = []
        unique_list = []
        total_list = []
        mean_count = (sum(sub_table.iloc[sample]["readCount"]) / 100)
        while counter < 100:
            temp = random.choices(sequences, weights=fraction, k = int(mean_count))  # randomly sample 100th of the sequences with the recalculated clonefractions as probablities
            temp_list.extend(temp)  ## extend the temp_list with the samples from each 'pull-out' of the 100 'pull-outs'
            unique = Counter(temp_list).keys()  ## count unique seequences from the increasing temp_list
            unique_list.append(unique)  ## for each run append the number of unique sequences to a new cell in this list - will be accumulative
            total_list.append(len(temp_list))  ## append the total sequences amount for plotting
            counter += 1
        plt.plot(total_list,
                 [len(x) for x in unique_list],
                 figure = fig,
                 label = sub_table.iloc[sample].Experiment,
                 alpha = 1,
                fillstyle = 'full',
                linewidth = 0.8,
        )
        x_axis.append(total_list)
        y_axis.append([len(x) for x in unique_list])
    if len(sub_table) > 7:
        warnings.warn("Risk of Overplotting: Many different colors are used. It may be hard to differ between the data")
  #  for i in range(len(x_axis)):
   #     plt.plot(x_axis[i], y_axis[i], **params_plot)
    plt.legend(sub_table.Experiment, title = "Sample Names", **legend_settings)
    plt.xlabel("Total sampled sequences", **font_settings)
    plt.ylabel("Total Unique Sequences", **font_settings)
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    plt.title("Sequencing depth of the given samples ",pad = 12, **font_settings)
    font_settings["fontsize"] = original_fontsize
    fig.tight_layout()




def calcFraction(df,x, target_column):
    return df[target_column] / df[target_column].sum()


def summarizeDuplicates(df, target_column, dependency):
    column = df[target_column].sum()
    df[target_column] = column
    df = df.drop_duplicates(dependency)
    return df

mapping_funcs = {
    "calcFraction": calcFraction,
    "summarizeDuplicates": summarizeDuplicates,
}



import pandas as pd
import os
import subprocess
import pickle
from glob import glob
import shutil
from ExpoSeq.augment_data.trimming import trimming
#from .trimming import trimming

#
def remove_directories(directory_path):
    # Iterate over all items in the directory
    for item in os.listdir(directory_path):
        shutil.rmtree(os.path.join(directory_path, item))


def collect_fastq_files(fastq_directory):
    filenames = []
    path_to_files = os.path.abspath(fastq_directory)
    filenames.extend(glob(os.path.join(path_to_files, "*.fastq")))
    if len(filenames) == 0:
       print(
            "You have not chosen a directory with fastq files.")

    else:
        print("These are the files you chose:")
        for i in filenames:
            print(os.path.basename(i))

    return filenames


def mixcr_(fastq_directory, path_to_mixcr, save_dir, paired_end_sequencing, threads, method, trim_div_by, trim_min_count, testing = False):
    if not testing:
        filenames = collect_fastq_files(fastq_directory)
    else:
        filenames = glob(os.path.join(r"C:\Users\nilsh\my_projects\ExpoSeq\tests", "*.fastq"))
    if len(filenames) == 0:
        print("You have not added any fastq files.")
        return
    print("Test Mixcr")
    subprocess.run(["java",
                    "-jar",
                    path_to_mixcr,
                    "--version"
                    ])
    confirmation = True
    if confirmation == True:
        if not os.path.isdir("temp"):
            os.mkdir("temp")

        module_dir = os.path.abspath("")
        sequencing_report = pd.DataFrame([])
        if paired_end_sequencing == True:
            filenames_unique_reverse = [unique for unique in filenames if "2.fastq" in unique]
            filenames_unique_forward = [unique for unique in filenames if "1.fastq" in unique]
            combined_filenames = []
            for i in filenames_unique_reverse:
                file_one = i
                basename = os.path.basename(os.path.splitext(file_one)[0])
                basename = basename[:len(basename) - 2]
                file_two = ""
                for j in filenames_unique_forward:
                    if basename in filenames_unique_forward:
                        file_two = j
                        break
                if file_two == "":
                    print("you are missing the the reverse strand for " + basename)
                else:
                    fastq_files = file_one + " " + file_two
                    combined_filenames.append(fastq_files)
            if len(combined_filenames) == 0:
                print("the system couldnt find any forward reverse matches. Please check your given directory or continue with single-end analysis.")
                return
        else:
            combined_filenames = filenames
        for filename in combined_filenames:
            try:
                basename = os.path.basename(os.path.splitext(filename)[0])
                basename = basename[:len(basename) - 2]
                filename_base = basename
                result = os.path.join(module_dir, "temp", filename_base + ".vdjca")
                clones = os.path.join(module_dir, "temp", basename + "clones.clns")
                subprocess.run(["java",
                                "-jar",
                                path_to_mixcr,
                                "align",
                                "-p " + method,
                                filename,
                                result,
                                "--report",
                                os.path.normpath(os.path.join(module_dir,
                                                              filename_base + "_AlignmentReport.txt")),
                                "--threads",
                                str(threads)
                                ])

                subprocess.run(["java",
                                "-jar",
                                path_to_mixcr,
                                "assemble",
                                "-OseparateByC=true",
                                "-OseparateByV=true",
                                "-OseparateByJ=true",
                                result,
                                clones,
                                "--threads",
                                str(threads)
                                ])

                subprocess.run(["java",
                                "-jar",
                                path_to_mixcr,
                                "exportClones",
                                "-cloneId",
                                "-readCount",
                                "-readFraction",
                                "-lengthOf CDR3",
                                "-nFeature CDR3",
                                "-aaFeature CDR3",
                                "-avrgFeatureQuality CDR3",
                                clones,
                                os.path.join(module_dir,
                                             "temp",
                                             basename + ".tsv"),
                                "--threads",
                                str(threads)
                                ])

                clones_sample = pd.read_table(os.path.join(module_dir,
                                                           "temp",
                                                           basename + "_IGH.tsv"))
                clones_sample = clones_sample[["cloneId",
                                               "readCount",
                                               "readFraction",
                                               "nSeqCDR3",
                                               "aaSeqCDR3",
                                               "minQualCDR3",
                                               "lengthOfCDR3",
                                               "meanQualCDR3"]]
                new_fractions = clones_sample.groupby("nSeqCDR3")["readFraction"].sum().reset_index()
                clones_sample = clones_sample.drop_duplicates(subset=["nSeqCDR3"], keep="first")
                clones_sample = new_fractions.merge(clones_sample,
                                                    how="left",
                                                    on="nSeqCDR3")
                clones_sample = clones_sample.sort_values(by="cloneId")
                clones_sample = clones_sample.reset_index()
                clones_sample = clones_sample.drop(columns=["readFraction_y", "index"])
                clones_sample = clones_sample.rename(columns={"readFraction_x": "cloneFraction"})
                clones_sample["Experiment"] = filename_base
                clones_sample = trimming(clones_sample,
                                         divisible_by=trim_div_by,
                                         min_count=trim_min_count,
                                         new_fraction="clonesFraction")
                sequencing_report = pd.concat([sequencing_report, clones_sample])
                files_to_remove = os.listdir(os.path.join(module_dir, "temp"))
                for file in files_to_remove:
                    os.remove(os.path.join(module_dir,
                                           "temp",
                                           file))
                report_dir = os.path.join(save_dir,
                                      "sequencing_report.txt")
                sequencing_report.to_csv(report_dir)
            except:
                remove_directories(os.path.join(module_dir, "temp"))

    else:
        print("The path to the mixcr jar file is not correct or you are missing the licence")


import os.path
from matplotlib.pyplot import savefig
from ast import literal_eval
import pkg_resources
try:
    import tkinter as tk
    from tkinter import filedialog
except:
    pass

def saveFig(name):
    pkg_path = pkg_resources.resource_filename("ExpoSeq", "")
    save_settings_file = os.path.join(pkg_path,
                                      "settings",
                                      "save_settings.txt")
    with open(save_settings_file, "r") as f:
        save_settings = f.read()
    save_settings = literal_eval(save_settings)
    if name == False:
        filename = input("how do you want to call the plot?")
    else:
        filename = name
    if save_settings["fname"] == "":
    #    print("Please choose a directory where you want to store your files")
        try:
            print("Choose the directory where you want to save the file with the filechooser")
            save_dir = filedialog.askdirectory()
        except:
            while True:
                save_dir = input("copy and paste the path to the directory where you want to save the files here.")
                if os.path.isdir(os.path.abspath(save_dir)):
                    break
                else:
                    print("Try again and input a valid directory")
        save_dir = os.path.abspath(save_dir)
        save_settings["fname"] = save_dir
        with open(save_settings_file, "w") as f:
            f.write(str(save_settings))
    else:
        pass
    filepath = save_settings['fname']
    form = save_settings['format']
    path = os.path.join(filepath, filename + "." + form)
    del save_settings['fname']
    savefig(path,
            **save_settings)


import matplotlib.pyplot as plt
from ast import literal_eval
import os
import pkg_resources
class PlotStyle:
    def __init__(self, ax, plot_type):
        self.pkg_path = pkg_resources.resource_filename("ExpoSeq", "")
        self.module_dir = os.path.abspath("")
        self.ax = ax
        font_settings_path = os.path.join(self.pkg_path,
                                          "settings",
                                          "font_settings.txt")
        with open(font_settings_path, "r") as f:
            font_settings = f.read()
        self.font_settings = literal_eval(font_settings)
        legend_settings_path = os.path.join(self.pkg_path,
                                            "settings",
                                            "legend_settings.txt")
        with open(legend_settings_path, "r") as f:
            legend_settings = f.read()
        self.legend_settings = literal_eval(legend_settings)
        self.plot_type = plot_type
    def title_figure(self, title):
        if self.plot_type == "single":
            self.ax.set_title(title)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def title_xaxis(self, xlabel):
        if self.plot_type == "single":
            self.ax.set_xlabel(xlabel)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def title_yaxis(self, ylabel):
        if self.plot_type == "single":
            self.ax.set_ylabel(ylabel)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def xaxis_limit(self, xmin, xmax):
        if self.plot_type == "single":
            self.ax.set_xlim(xmin, xmax)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def yaxis_limit(self, ymin, ymax):
        if self.plot_type == "single":
            self.ax.set_ylim(ymin, ymax)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def xscale(self, ticks):
        if self.plot_type == "single":
            self.ax.set_xticks(ticks)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def yscale(self, ticks):
        if self.plot_type == "single":
            self.ax.set_yticks(ticks)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def xticklabels(self, labels):
        if self.plot_type == "single":
            self.ax.set_xticklabels(labels)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def yticklabels(self, labels):
        if self.plot_type == "single":
            self.ax.set_yticklabels(labels)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def xaxis_log(self):
        if self.plot_type == "single":
            self.ax.set_xscale("log")
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def yaxis_log(self):
        if self.plot_type == "single":
            self.ax.set_yscale("log")
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def xaxis_linear(self):
        if self.plot_type == "single":
            self.ax.set_xscale("linear")
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def yaxis_linear(self):
        if self.plot_type == "single":
            self.ax.set_xscale("linear")
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def grid(self, visible=True):
        if self.plot_type == "single":
            self.ax.grid(visible)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def style(self, style):
        if self.plot_type == "single":
            plt.style.use(style)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def facecolor(self, color):
        if self.plot_type == "single":
            self.ax.set_facecolor(color)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")

    def annotate_coordinate(self, x, y, text, **kwargs):
        if self.plot_type == "single":
            self.ax.annotate(text,
                         xy=(x, y),
                         xytext=(x, y),
                         **kwargs)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def annotate_coordinate_with_arrow(self, x, y, text, **kwargs):
        if self.plot_type == "single":
            self.ax.annotate(text,
                         xy=(x, y),
                         xytext=(x, y),
                         arrowprops=dict(facecolor='red', arrowstyle="->"),
                         **kwargs)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def font_family(self, font_name, global_change = False):
        if self.plot_type == "single":
            plt.rc('font',
                family=font_name)
            if global_change == True:
                self.font_settings["family"] = font_name
        else:
            print(
                "Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def font_style(self, style, global_change = False):
        if self.plot_type == "single":
            plt.rc(weight = style)
            if global_change == True:
                self.font_settings["fontweight"] = style
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def font_color(self, color):
        if self.plot_type == "single":
            for item in ([self.ax.title, self.ax.xaxis.label, self.ax.yaxis.label] +
                         self.ax.get_xticklabels() + self.ax.get_yticklabels()):
                item.set_color(color)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")
    def titlesize_figure(self, size):
        if self.plot_type == "single":
            self.ax.title.set_size(size)
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")

    def xy_titlesize(self, size, global_change = False):
        if self.plot_type == "single":
            self.ax.xaxis.label.set_size(size)
            self.ax.yaxis.label.set_size(size)
            if global_change == True:
                self.font_settings["fontsize"] = size
        else:
            print("Unfortunately, you can't change the layout of subplots. You can try to change it using the matplotlib.pyplot package")



import matplotlib
import matplotlib.pyplot as plt
import seaborn as sns
from ExpoSeq.tidy_data.heatmaps import tidy_jaccard, tidy_sorensen, tidy_morosita_horn, tidy_heatmap_share

import numpy as np

def plot_heatmap(sequencing_report, protein, heatmap, ax, colorbar_settings, font_settings, specific_experiments = False):

    if heatmap == "morosita_horn":
        unique_sequences, unique_experiments = tidy_morosita_horn.cleaning_data(sequencing_report,
                                                                                protein = protein,
                                                                                specific_experiments = specific_experiments)
        matrix, unique_sequences, unique_experiments = tidy_morosita_horn.morosita_horn_matrix(unique_sequences,
                                                                                               unique_experiments)
        title = "Identity between samples based on Morosita Horn index"
    if heatmap == "jaccard":
        matrix, unique_sequences, unique_experiments = tidy_jaccard.cleaning_jaccard(sequencing_report,
                                                                                     protein=protein,
                                                                                     specific_experiments = specific_experiments)
        title = "Identity between samples based on Jaccard index"
    if heatmap == "sorensen":
        matrix, unique_sequences, unique_experiments = tidy_sorensen.heatmap_sorensen(sequencing_report,
                                                                                      protein = protein,
                                                                                      specific_experiments = specific_experiments)
        title = "Identity between samples based on Sorensen index"
    if heatmap == "relative":
        matrix, unique_sequences, unique_experiments = tidy_heatmap_share.heatmap_share(sequencing_report,
                                                                                        protein=protein,
                                                                                        specific_experiments = specific_experiments)
        title = "Identity between samples based on relative coefficient"

    unique_experiments = list(unique_experiments)
    matrix = matrix.sort_index(axis=1)
    matrix = matrix.sort_index(axis = 0)
    matplotlib.use('Qt5Agg')
    colorbar_sets = {**colorbar_settings,
                         **{'label': "Degree of Identity"}}
    sns.heatmap(matrix,
                ax = ax,
                cbar_kws = colorbar_sets)

    plt.xticks(ticks = np.arange(0.5, len(unique_experiments) + 0.5 ,1),
               labels = unique_experiments,
               rotation = 45,
               ha = 'right',
               size = 12) # create a function which finds the perfect size based on counts of xlabels
    plt.yticks(ticks = np.arange(0.5, len(unique_experiments) + 0.5 ,1),
               labels = unique_experiments,
               rotation = 0,
               va='center',
               size=12)
    original_fontsize = font_settings["fontsize"]
    font_settings["fontsize"] = 22
    ax.set_title(title,pad = 12, **font_settings)
    font_settings["fontsize"] = original_fontsize









import pandas as pd


def cleaning(sample_name, report, chosen_seq_length):
    sample = report[report["Experiment"] == sample_name]
    local_report = sample[["Experiment", "clonesFraction", "aaSeqCDR3"]]
    sequences = local_report.aaSeqCDR3
    #local_report = local_report.sort_values("cloneFraction").groupby("Experiment", as_index = False).head(3) # filters out three highest values of each group
    aminoacids = "ACDEFGHIKLMNPQRSTVWY"
    if chosen_seq_length == "median":
        chosen_sequence = int(sequences.str.len().median())#raise error if 0
    if type(chosen_seq_length) == int:
        chosen_sequence = int(chosen_seq_length)
    if chosen_seq_length == "max":
        chosen_sequence = int(sequences.str.len().max())
    if chosen_seq_length == "mean":
        chosen_sequence = int(sequences.str.len().mean())
    compDict = {aa: chosen_sequence*[0] for aa in aminoacids}
    sequences = local_report[local_report.aaSeqCDR3.astype(str).str.len() == chosen_sequence]["aaSeqCDR3"]
    length_filtered_seqs = sequences.shape[0]
    for seq in sequences:
        for aa_position in range(len(seq)):
            aminoacid = seq[aa_position]
            if aminoacid == '*':
                pass
            else:
                compDict[aminoacid][aa_position] += 1
    aa_distribution = pd.DataFrame.from_dict(compDict)
    #aa_distribution = aa_distribution.divide(aa_distribution.shape[1])
    aa_distribution = aa_distribution.divide(aa_distribution.sum(axis = 1), axis = 0)
    return aa_distribution, chosen_sequence, length_filtered_seqs






def check_completeness(all_alignment_reports, sequencing_report_all):
    all_alignment_reports = all_alignment_reports.sort_values("Input file(s)").reset_index(drop=True)
    unique_experiments = sequencing_report_all.sort_values("Experiment")["Experiment"].unique()
    unique_experiments = list(unique_experiments)
    not_found_strings = []
    for i in all_alignment_reports.iloc[:, 0]:
        found = False
        for j in unique_experiments:
            if j in i:
                found = True
        if found == False:
            print("Your data is imcomplete for the following sample: " + j)
            not_found_strings.append(j)


    for string in not_found_strings:
        all_alignment_reports = all_alignment_reports[~all_alignment_reports.iloc[:, 0].str.contains(string, na=False)]

    return all_alignment_reports


from ExpoSeq.full_pipe import run_pipeline
from ExpoSeq.pipeline import PlotManager

def run_automatic(test = False, plotmanager = PlotManager()):
    if not test:
        run_pipeline(plot = plotmanager, test = test)
    else:
        run_pipeline(plot = plotmanager, test = True)


from ExpoSeq.tidy_data.interpret_data import add_fraction


def cleaning_data(sequencing_report, samples):
    sub_table = sequencing_report.loc[sequencing_report["Experiment"].isin(samples)]
    sub_table = add_fraction(sequencing_report = sub_table)
    apply_fun = lambda x: list(x)
    sub_table = sub_table.groupby("Experiment")[['nSeqCDR3', 'clonesFraction', 'readCount']].agg(apply_fun).reset_index()
    return sub_table




from .pipeline import PlotManager
import os
import pandas as pd
import shutil
import pkg_resources
from ast import literal_eval

def run_pipeline(antigens = None, usq_multiple = "n", pass_interrupt = True, test = False):
    pkg_path = pkg_resources.resource_filename("ExpoSeq", "")
    save_settings_path = os.path.join(pkg_path,
                                      "settings",
                 "save_settings.txt")
    with open(save_settings_path, "r") as f:
        save_settings = f.read()
    save_settings = literal_eval(save_settings)
    original_path = save_settings["fname"]
    if antigens == True or antigens == False or antigens == None:
        pass
    else:
        raise ValueError("Enter the antigens in a list and seperate them by comma")
    if usq_multiple in ["Y", "y", "n", "N"]:
        pass
    else:
        raise ValueError("Invalid value. Please enter Y or n")
    if pass_interrupt in [True, False]:
        pass
    else: 
        raise ValueError("Invalid value. Please enter True or False")

    
    if not os.path.isdir("results"):
        os.mkdir("results")
    else:
        pass
    result_dir = os.path.abspath("results")

    plot = PlotManager(test_version = test)
    folder_name = plot.experiment
    experiment_name = folder_name
    if not test or not os.path.isdir(os.path.join(result_dir, "Test")):
        os.mkdir(os.path.join(result_dir, folder_name))
        os.mkdir(os.path.join(result_dir, folder_name, "Quality"))
    assert os.path.isdir(os.path.join(result_dir,
                                      folder_name,
                                      "Quality")), "The directory could not be created"
    save_path = os.path.abspath(os.path.join(result_dir,
                                             folder_name,
                                             "Quality"))
    plot.settings_saver.change_save_path(path = save_path), "The save path could not be changed"

    plot.lengthDistribution_multi()
    if test:
        print("length Distribution successfully created")
    plot.save(name = "length_Distribution" + folder_name)
    plot.logoPlot_multi()
    if test:
        print("Logo Plot successfully created")
    plot.save(name = "logo Plots " + folder_name)
    usq_multiple = usq_multiple
    unique_experiments = list(plot.unique_experiments.values())
    if usq_multiple.lower() in ["Y", "y"]:
        more_plots = True
        while more_plots == True:
            plot.print_samples()
            sample_names = input("Choose the sample from the list above which you want to include. Please seperate your input with commas")
            
            if not type(sample_names) == list:
                sample_names = sample_names.split(",")
                sample_names = [value.replace("'", "").replace('"', '') for value in sample_names]
                sample_names = [value.replace(" ", "") for value in sample_names]
                sample_names = [value for value in sample_names if value]
            else:
                pass
            for value in sample_names:
                if value not in unique_experiments and pass_interrupt == False:
                    interrupt = input("The system could not find " + value + " in your sample name. If you want to try again, press Y. Otherwise the analysis will be conducted without this value")
                    while True:
                        if interrupt.lower() in ["Y", "y"]:
                            break
                        else:
                            sample_names.remove(value)
                
            plot.usqPlot(samples = sample_names)
            while True:
                usq_name = input("How do you want to call your USQ Plot?")
                if not os.path.isfile(os.path.join(save_path, usq_name)):
                    plot.save(usq_name)
                    break
                else:
                    print("The file already exists. Please try another value.")
            while True:
                more_plots = input("Do you want to create more USQ Plots with different samples? (Y/n)")
                if more_plots.lower() in ["Y", "y", "n", "N"]:
                    break
                else:
                    print("Please enter a valid value")
            if more_plots.lower() in ["Y", "y"]:
                more_plots = True
                unique_experiments = [value for value in unique_experiments if value not in sample_names]
            else:
                more_plots = False
                break
    else:
        for experiment in unique_experiments:
            try:
                plot.usqPlot(samples = [experiment])

                plot.save("Usq Plot " + experiment)
            except:
                print("A problem appeard for " + experiment + ". The USQ plot could not be created")
    unique_experiments = list(plot.unique_experiments.values())
    plot.morosita_horn()
    if test:
        print("Morosita Horn successfully created")
    plot.save("morosita_horn_identity")
    plot.jaccard()
    if test:
        print("Jaccard successfully created")
    plot.save("jaccard_identity")
    plot.sorensen()
    if test:
        print("Sorensen Dice successfully created")
    plot.save("Sorensen_Dice_identity")
    plot.relative()
    if test:
        print("Relative Identity successfully created")
    plot.save("Relative_Identity")

    os.mkdir(os.path.join(result_dir,
                          folder_name,
                          "Sequence Analysis"))
    assert os.path.isdir(os.path.join(result_dir,folder_name, "Sequence Analysis")), "The directory : " + os.path.join(result_dir,folder_name, "Sequence Analysis") + " could not be created"
    for experiment in unique_experiments:
        try:
            os.mkdir(os.path.join(result_dir,folder_name, "Sequence Analysis", experiment))
            save_path = os.path.abspath(os.path.join("results", folder_name, "Sequence Analysis", experiment))
            plot.settings_saver.change_save_path(path = save_path)
            plot.logoPlot_single(experiment)
            print("Logo Plot for " + experiment + " created")
            plot.save("Logo Plot for " + experiment)
            plot.rel_seq_abundance(samples = [experiment])
            print("Relative Sequence Abundance for " + experiment + " created")
            plot.save("Sequence Abundance for " + experiment)
     #      plot.basic_cluster(experiment)
      #      plot.save("Clustering based on Levenshtein Distance")
            try:
                plot.aa_distribution(experiment, region = [3, 8])
                plot.save("AA Distribution " + "in region 3 - 8")
                print("AA Distribution for " + experiment + " created")
            except:
                print("sequence logo plot for " + experiment + "in region: "+ "3, 8" + " could not be created")
            try:
                plot.aa_distribution(sample = experiment, region = [9, 14])
                
                plot.save("AA Distribution "+ "in region 9 - 14")
            except: 
                print("sequence logo plot for " + experiment + "in region: "+ "9, 14" + " could not be created")
            try:
                plot.aa_distribution(sample = experiment, region = [15, 20])
                plot.save("AA Distribution " + "in region 15 - 20")
            except:
                print("sequence logo plot for " + experiment + "in region: "+ "15, 20" + " could not be created")

        except: pass

    if isinstance(plot.binding_data, pd.DataFrame) == True:
        print("You have binding data for the following Antigens")
        plot.print_antigens()
        all_antigens = plot.binding_data.columns.to_list()[1:-1]
        if test != True and antigens == None:
            count_AG = input("If you want to create a TSNE Embedding for all Antigens press 1. If you want to select certain Antigens, press 2.")
            while True:
                if count_AG in ["1", "2"]:
                    break
                else:
                    print("Please enter a valid value")
        else:
            count_AG = "1"
        if count_AG == str(1):
            antigens = all_antigens
        else:
           
            if not type(antigens) == list:
                antigens = antigens.split(",")
                antigens = [value.replace("'", "").replace('"', '') for value in antigens]
                antigens = [value.replace(" ", "") for value in antigens]
                assert type(antigens) == list, "The antigens have to be in a list. The program could not solve this error"
            else:
                pass
            for value in antigens:
                if value not in all_antigens:
                    while True:
                        interrupt = input("The system could not find " + value + " in your sample name. If you want to try again, press Y. Otherwise press n and the analysis will be conducted without this value")
                        if interrupt in ["Y", "y", "n", "N"]:
                            break
                        else:
                            print("Please enter a valid value")
                    if interrupt.lower() in ["Y", "y"] and not test:
                        break
                    else:
                        antigens.remove(value)
                            
        for antigen in antigens:
            os.mkdir(os.path.join(result_dir, folder_name, "Binding Analysis"))
            os.mkdir(os.path.join(result_dir, folder_name, "Binding Analysis", antigen))
            save_path = os.path.abspath(os.path.join("results", folder_name, "Binding Analysis", antigen))
            for experiment in unique_experiments:
                plot.tsne_cluster_AG(sample = experiment, toxins = [antigen], toxin_names = False)
                if test:
                    break

    else:
        print("You have not given any binding data. So this part of the analysis has not been carried out.")
    if test:
        shutil.rmtree(os.path.join(result_dir, folder_name))

    plot.settings_saver.change_save_path(path = original_path)
    
    
    



from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name = "ExpoSeq",
    version = "1.1.14",
    description = "A pacakge which provides various ways to analyze NGS data from phage display campaigns.",
    long_description=long_description,
    long_description_content_type = "text/markdown",
    url = "https://github.com/nilshof01/ExpoSeq",
    author = "Nils Hofmann",
    author_email = "n.hofmann.99@web.de",
    license = "Apache License 2.0",
    package_data={
        "ExpoSeq": ["settings/*.txt", "test_data/test_files/*", "test_data/*"]
    },
    install_requires = [
                        "numpy>=1.23.5",
                        "pandas>=1.5.3",
                        "matplotlib>=3.6.3",
                        "scipy>=1.10.0",
                        "seaborn>=0.12.2",
                        "logomaker==0.8",
                        "editdistance==0.6.2",
                        "networkx==2.6.3",
                        "PyQt5==5.15.7",
                        "scikit-learn>=1.2.1",
                        "biopython==1.80",
                        "sgt>=2.0.3",
                        ],
    python_requires=">=3.8",
    package_dir = {"": "src"},
    packages = find_packages(where="src"),

)



import shutil
import os
from ast import literal_eval
import pkg_resources
def remove_directories(directory_path):
    # Iterate over all items in the directory
    for item in os.listdir(directory_path):
        shutil.rmtree(os.path.join(directory_path, item))

def original_settings():
    pkg_path = pkg_resources.resource_filename("ExpoSeq", "")
    module_dir = os.path.abspath("")
    experiments_dir = os.path.join(module_dir,
                                   "my_experiments")
    remove_directories(experiments_dir)
    common_vars_path = os.path.join(pkg_path,
                                    "settings",
                                    "global_vars.txt")
    with open(common_vars_path, "r") as f:
        global_params = f.read()
    global_params = {'mixcr_path': '', 'last_experiment': '', 'api_gpt3': ''}
    with open(common_vars_path, "w") as f:
        f.write(str(global_params))
    save_path = os.path.join(pkg_path,
                             "settings",
                             "save_settings.txt")
    save_params = {'fname': '', 'format': 'png', 'dpi': 300}
    with open(save_path, "w") as f:
        f.write(str(save_params))
    legend_settings_path = os.path.join(pkg_path,
                                        "settings",
                                        "legend_settings.txt")
    legend_params = {"loc": "upper right", #'upper left', 'upper right', 'lower left', 'lower right'
                    "bbox_to_anchor": (1, 1),
                    "ncols": 1,
                    "fontsize": "medium", #'xx-small', 'x-small', 'small', 'medium', 'large', 'x-large', 'xx-large'
                    "frameon": True,
                    "framealpha": 1,
                    "facecolor": "white", #background color
                    "mode": None,
                    "title_fontsize": 'small',
                    }
    with open(legend_settings_path, "w") as f:
        f.write(str(legend_params))
    font_settings_path = os.path.join(pkg_path,
                                      "settings",
                                      "font_settings.txt")
    font_settings_params = {
                            "fontfamily": "serif",
                            "fontsize": "14",
                            "fontstyle": "normal",
                            "fontweight": "bold",
                            #rotation
                            }
    with open(font_settings_path, "w") as f:
        f.write(str(font_settings_params))
    colorbar_path = os.path.join(pkg_path,
                                 "settings",
                                 "colorbar.txt")
    colorbar_params = {
                        "cmap": "inferno",
                        "orientation": "vertical",
                        "spacing": "proportional",
                        "extend": "neither"
                        }
    with open(colorbar_path, "w") as f:
        f.write(str(colorbar_params))







from scipy.sparse import csr_matrix
import numpy as np
import editdistance
import networkx as nx

def cleaning(report, max_ld, min_ld):
    aa = list(report["aaSeqCDR3"])
    num_strings = len(aa)
    string_index = [*range(0, num_strings, 1)]
    max_ld = max_ld
    min_ld = min_ld
    collect_coordinates = np.array([0, 0, 0]).reshape(1,3)
    for i in range(len(string_index)):
        string1 = aa[i]
        index_1 = i
        for index in range(index_1):
            string2 = aa[index]
            distance = editdistance.distance(string1, string2)
            if distance>= min_ld and distance <= max_ld:
                collect_coordinates = np.concatenate((collect_coordinates,
                                                      np.array([index_1, index, distance]).reshape(1, 3)))
    mat = csr_matrix((collect_coordinates[:, 2],
                      (collect_coordinates[:, 0], collect_coordinates[:, 1])),
                     (len(aa), len(aa)))
    G = nx.convert_matrix.from_scipy_sparse_matrix(mat)
    degree_sequence = sorted([d for n, d in G.degree()], reverse=True)
    return G, degree_sequence

